@@ -28,107 +28,107 @@ namespace {
   // optimization to pre-compute the logs of the first 100 integers.
   inline double log_integer(const int d) {
     switch (d) {
-      case 1: return log(1);
-      case 2: return log(2);
-      case 3: return log(3);
-      case 4: return log(4);
-      case 5: return log(5);
-      case 6: return log(6);
-      case 7: return log(7);
-      case 8: return log(8);
-      case 9: return log(9);
-      case 10: return log(10);
-      case 11: return log(11);
-      case 12: return log(12);
-      case 13: return log(13);
-      case 14: return log(14);
-      case 15: return log(15);
-      case 16: return log(16);
-      case 17: return log(17);
-      case 18: return log(18);
-      case 19: return log(19);
-      case 20: return log(20);
-      case 21: return log(21);
-      case 22: return log(22);
-      case 23: return log(23);
-      case 24: return log(24);
-      case 25: return log(25);
-      case 26: return log(26);
-      case 27: return log(27);
-      case 28: return log(28);
-      case 29: return log(29);
-      case 30: return log(30);
-      case 31: return log(31);
-      case 32: return log(32);
-      case 33: return log(33);
-      case 34: return log(34);
-      case 35: return log(35);
-      case 36: return log(36);
-      case 37: return log(37);
-      case 38: return log(38);
-      case 39: return log(39);
-      case 40: return log(40);
-      case 41: return log(41);
-      case 42: return log(42);
-      case 43: return log(43);
-      case 44: return log(44);
-      case 45: return log(45);
-      case 46: return log(46);
-      case 47: return log(47);
-      case 48: return log(48);
-      case 49: return log(49);
-      case 50: return log(50);
-      case 51: return log(51);
-      case 52: return log(52);
-      case 53: return log(53);
-      case 54: return log(54);
-      case 55: return log(55);
-      case 56: return log(56);
-      case 57: return log(57);
-      case 58: return log(58);
-      case 59: return log(59);
-      case 60: return log(60);
-      case 61: return log(61);
-      case 62: return log(62);
-      case 63: return log(63);
-      case 64: return log(64);
-      case 65: return log(65);
-      case 66: return log(66);
-      case 67: return log(67);
-      case 68: return log(68);
-      case 69: return log(69);
-      case 70: return log(70);
-      case 71: return log(71);
-      case 72: return log(72);
-      case 73: return log(73);
-      case 74: return log(74);
-      case 75: return log(75);
-      case 76: return log(76);
-      case 77: return log(77);
-      case 78: return log(78);
-      case 79: return log(79);
-      case 80: return log(80);
-      case 81: return log(81);
-      case 82: return log(82);
-      case 83: return log(83);
-      case 84: return log(84);
-      case 85: return log(85);
-      case 86: return log(86);
-      case 87: return log(87);
-      case 88: return log(88);
-      case 89: return log(89);
-      case 90: return log(90);
-      case 91: return log(91);
-      case 92: return log(92);
-      case 93: return log(93);
-      case 94: return log(94);
-      case 95: return log(95);
-      case 96: return log(96);
-      case 97: return log(97);
-      case 98: return log(98);
-      case 99: return log(99);
-      case 100: return log(100);
-      default: return log(d);
+      case 1: return log(1.0);
+      case 2: return log(2.0);
+      case 3: return log(3.0);
+      case 4: return log(4.0);
+      case 5: return log(5.0);
+      case 6: return log(6.0);
+      case 7: return log(7.0);
+      case 8: return log(8.0);
+      case 9: return log(9.0);
+      case 10: return log(10.0);
+      case 11: return log(11.0);
+      case 12: return log(12.0);
+      case 13: return log(13.0);
+      case 14: return log(14.0);
+      case 15: return log(15.0);
+      case 16: return log(16.0);
+      case 17: return log(17.0);
+      case 18: return log(18.0);
+      case 19: return log(19.0);
+      case 20: return log(20.0);
+      case 21: return log(21.0);
+      case 22: return log(22.0);
+      case 23: return log(23.0);
+      case 24: return log(24.0);
+      case 25: return log(25.0);
+      case 26: return log(26.0);
+      case 27: return log(27.0);
+      case 28: return log(28.0);
+      case 29: return log(29.0);
+      case 30: return log(30.0);
+      case 31: return log(31.0);
+      case 32: return log(32.0);
+      case 33: return log(33.0);
+      case 34: return log(34.0);
+      case 35: return log(35.0);
+      case 36: return log(36.0);
+      case 37: return log(37.0);
+      case 38: return log(38.0);
+      case 39: return log(39.0);
+      case 40: return log(40.0);
+      case 41: return log(41.0);
+      case 42: return log(42.0);
+      case 43: return log(43.0);
+      case 44: return log(44.0);
+      case 45: return log(45.0);
+      case 46: return log(46.0);
+      case 47: return log(47.0);
+      case 48: return log(48.0);
+      case 49: return log(49.0);
+      case 50: return log(50.0);
+      case 51: return log(51.0);
+      case 52: return log(52.0);
+      case 53: return log(53.0);
+      case 54: return log(54.0);
+      case 55: return log(55.0);
+      case 56: return log(56.0);
+      case 57: return log(57.0);
+      case 58: return log(58.0);
+      case 59: return log(59.0);
+      case 60: return log(60.0);
+      case 61: return log(61.0);
+      case 62: return log(62.0);
+      case 63: return log(63.0);
+      case 64: return log(64.0);
+      case 65: return log(65.0);
+      case 66: return log(66.0);
+      case 67: return log(67.0);
+      case 68: return log(68.0);
+      case 69: return log(69.0);
+      case 70: return log(70.0);
+      case 71: return log(71.0);
+      case 72: return log(72.0);
+      case 73: return log(73.0);
+      case 74: return log(74.0);
+      case 75: return log(75.0);
+      case 76: return log(76.0);
+      case 77: return log(77.0);
+      case 78: return log(78.0);
+      case 79: return log(79.0);
+      case 80: return log(80.0);
+      case 81: return log(81.0);
+      case 82: return log(82.0);
+      case 83: return log(83.0);
+      case 84: return log(84.0);
+      case 85: return log(85.0);
+      case 86: return log(86.0);
+      case 87: return log(87.0);
+      case 88: return log(88.0);
+      case 89: return log(89.0);
+      case 90: return log(90.0);
+      case 91: return log(91.0);
+      case 92: return log(92.0);
+      case 93: return log(93.0);
+      case 94: return log(94.0);
+      case 95: return log(95.0);
+      case 96: return log(96.0);
+      case 97: return log(97.0);
+      case 98: return log(98.0);
+      case 99: return log(99.0);
+      case 100: return log(100.0);
+      default: return log(static_cast<double>(d));
     }
   }
 } // namespace
